#include "svlVision.h"
//#include "svlDevel.h"
#include "svlISMpatch.h"
#include <iomanip>

svlISMpatch::svlISMpatch(const CvMat * rawPixels) : _rawPixels(rawPixels)
{
  double m = cvMean(_rawPixels);
  
  _deviations = cvCreateMat(_rawPixels->rows, _rawPixels->cols, CV_32FC1);

  CvScalar negM = cvScalar(-m);

  cvAddS(_rawPixels,negM, _deviations);

  _denomFactor = 0;

  for (int i = 0; i < _deviations->rows; i++)
    {
      for (int j = 0; j < _deviations->cols; j++)
	{
	  double deviation = cvmGet(_deviations,i,j);
	  _denomFactor += deviation * deviation;
	}
    }

  _denomFactor = sqrt(_denomFactor);
}

double svlNGC(const svlISMpatch * a, const svlISMpatch * b)
{
  double rval = cvDotProduct(a->_deviations, b->_deviations);

  rval /= (a->_denomFactor * b->_denomFactor);

  assert(rval >= -1.0);
  
  if (rval > 1.0)
    {
      if (rval > 1.000000000001)
	{
	  cout << "NGC value of "<<setprecision(34) << rval<<endl;
	  assert(false);
}	

      rval = 1.0;
    }

  return rval;
}

svlISMpatch * svlMerge(const svlISMpatch * a, const svlISMpatch * b)
{
  CvMat * m = cvCreateMat(a->_rawPixels->rows, a->_rawPixels->cols, CV_32FC1);
  cvmAdd(a->_rawPixels, b->_rawPixels, m);
  cvScale(m,m,0.5);

  return new svlISMpatch(m);
}

svlISMpatch::~svlISMpatch()
{
  cvReleaseMat((CvMat **) &_rawPixels);
  cvReleaseMat((CvMat **) &_deviations);
}

void svlISMpatch::writePixels(ofstream & ofs) const
{
  unsigned counter = 0;
  for (unsigned i = 0; i < (unsigned)_rawPixels->rows; i++)
    {
      for (unsigned j = 0; j < (unsigned)_rawPixels->cols; j++)
	{
	  ofs << cvmGet(_rawPixels,i,j);

	  counter++;

	  if (counter == 4)
	    {
	      ofs << "\n";
	      counter = 0;
	    }
	  else
	    ofs << "\t";
	}
    }
}

bool svlISMpatch::operator!=(const svlISMpatch & o) const
{
  if (_rawPixels->rows != o._rawPixels->rows)
    return true;

  if (_rawPixels->cols != o._rawPixels->cols)
    return true;

  for (int i = 0; i < _rawPixels->rows; i++)
    {
      for (int j = 0; j < _rawPixels->cols; j++)
	{

	  if (! svlFloatCompare(cvmGet(_rawPixels,i,j), cvmGet(o._rawPixels,i,j)))
	    return true;
	}
    }

  return false;
}

void svlISMpatch::draw(IplImage * img, unsigned ulx, unsigned uly) const
{
  for (int i = 0; i < _rawPixels->rows; i++)
    {
      for (int j = 0; j < _rawPixels->cols; j++)
	{
	  double val = cvmGet(_rawPixels, i,j );
	  CvScalar color = cvScalar(255.0 *  val, 255.0*val, 255.0*val);

	  cvSet2D(img,uly+i,ulx+j,color);
	}
    }
}
