//Author: Ian Goodfellow
//File: svlISMpatch.h
//Description: A class for storing an image patch, optimized for use with
//ISM. Specifically, it stores intermediate computations for calculating
//the NGC, making that operation faster

#pragma once


class svlISMpatch
{
 public:
  friend double svlNGC(const svlISMpatch * a, const svlISMpatch * b);
  friend svlISMpatch * svlMerge(const svlISMpatch * a, const svlISMpatch * b);

  svlISMpatch(const CvMat * rawPixels);//the class will become the owner of rawpixels
  ~svlISMpatch();

  void writePixels(ofstream & ofs) const;
  bool operator!=(const svlISMpatch & o) const;
  void draw(IplImage * img, unsigned ulx, unsigned uly) const;

 protected:
  const CvMat * _rawPixels;
  CvMat * _deviations;
  double _denomFactor;
};

double svlNGC(const svlISMpatch * a, const svlISMpatch * b);
//Does not free the originals
svlISMpatch * svlMerge(const svlISMpatch * a, const svlISMpatch * b);
