/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlKdTreeT.cpp
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
**
** DESCRIPTION:
**   Implementation file for the templated kd-tree class.
*****************************************************************************/

#include "svlKdTreeT.h"

template <>
int svlKdTreeT<svlPoint2d>::select_best_median(vector<int> &ix, int left, int right, int &dim, double &span) {
	svlPoint2d min_pt = (*_P)[ix[left]], max_pt = (*_P)[ix[left]];
	for ( int i=left+1; i<=right; ++i ) {
		min_pt.min_with((*_P)[ix[i]]);
		max_pt.max_with((*_P)[ix[i]]);
	}
	max_pt -= min_pt;
	span = max_pt.pt_max(); // save the max separation
	dim = max_pt.x > max_pt.y ? 0 : 1;
	return select_median(ix, left, right, ((right-left)>>1)+left, dim);
}

template <>
int svlKdTreeT<svlPoint3d>::select_best_median(vector<int> &ix, int left, int right, int &dim, double &span) {
	svlPoint3d min_pt = (*_P)[ix[left]], max_pt = (*_P)[ix[left]];
	for ( int i=left+1; i<=right; ++i ) {
		min_pt.min_with((*_P)[ix[i]]);
		max_pt.max_with((*_P)[ix[i]]);
	}
	max_pt -= min_pt;
	span = max_pt.pt_max(); // save the max separation
	if ( max_pt.x > max_pt.y )
		if ( max_pt.x > max_pt.z )
			dim = 0;
		else
			dim = 2;
	else
		if ( max_pt.y > max_pt.z )
			dim = 1;
		else
			dim = 2;
	return select_median(ix, left, right, ((right-left)>>1)+left, dim);
}

