/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlPlane.h
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
** DESCRIPTION:
**   Class for encapsulating two svlPoint3d's for clustering centroids.
*****************************************************************************/

#pragma once

#include "cv.h"
#include "cxcore.h"
#include <vector>

//#include "svlDevel.h"
//#include "svlVision.h"
#include "svlAlgorithms.h"
#include "svlPoint2d.h"
#include "../vision/svlPoint3d.h"
#include "svlSortVectorDI.h"

using namespace std;

class svlPlane {
public:
	double off; // offset along direction of normal vector
	svlPoint3d p, n, u1, u2; // point centroid, normal vector, and first two singular vectors
	svlPoint2d min_pt2, max_pt2; // bounding points in projection space
	bool flag;
	vector<svlPoint2d> Hull2d; // 2D points along hull w.r.t. u1 and u2
	vector<svlPoint3d> Hull3d, HullPs; // 3D points along hull in global coordinates; normals to hull segments
	vector<double> hullLims;
	static svlAlgorithms _sva;

public:
	svlPlane();
	svlPlane(double val);
	svlPlane(double p0, double p1, double p2, double n0, double n1, double n2);
	svlPlane(const svlPoint3d& p, const svlPoint3d& n);
	virtual ~svlPlane();

	svlPlane& set(double val);
	svlPlane& set(const svlPlane& pl);
	// svlPlane& set(const svlPoint3d& p_in, const svlPoint3d& n_in);
	// svlPlane& add(const svlPoint3d& p_in, const svlPoint3d& n_in);

	double norm() const;
	double norm2() const;
	double perp_distance(const svlPoint3d& pt) const;
	double dist(const svlPoint3d& pt) const;
	void offset(const svlPoint3d &t);
	void rotateX(double rad);
	svlPlane& operator/=(const double d);
	friend svlPoint3d operator-(const svlPlane& pl);

	bool isVertical();
	bool isHorizontal();
	svlPoint2d project(const svlPoint3d &pt) const;
	svlPoint3d project3d(const svlPoint3d &pt) const;
	svlPoint3d intercept3d(svlPoint3d ray) const;
	double interceptScale(svlPoint3d ray) const;
	svlPoint3d& interceptThis(svlPoint3d &ray);
	double minAbsPerpDist(const vector<svlPoint3d> &P);
	void heightMeanVar(const vector<svlPoint3d> &P, double &mean, double &var);
	void heightMeanVar(const vector<svlPoint3d> &P, const vector<int> &ix, double &mean, double &var);
	bool supports(svlPoint3d pt, double &dist);
	bool supports(svlPoint3d pt, double &dist, double &indist);
	double percentSupport(vector<svlPoint3d> &P);
	double percentSupport(vector<svlPoint3d> &P, vector<int> &ix);

	// serious methods
	void computeHulls(const vector<svlPoint3d> &Pts, const vector<int> &ix);
	void computeHulls(const vector<svlPoint3d> &Pts, const vector<int> &ix,
					  CvMat* P, CvMat* P2, CvMat* PPT, CvMat* U, CvMat* S, CvMat* l, CvMat* U12, CvMat* minmax, CvMemStorage* storage);
	bool insideHull(svlPoint3d pt);
	void trimHull();
	void trimHull(int h_size);
	//void meanShiftCorrect(const vector<svlPoint3d> &P, const vector<svlPoint3d> &N, const vector<int> &ix);
};

