/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlSortVectorDI.h
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
** DESCRIPTION:
**   Indexed vector for sorting and recovering the permutation. Similar to
**   MATLAB [vals_sorted,idx] = sort(vals);
*****************************************************************************/

#pragma once

#include <functional>
#include <vector>
#include <iostream>
#include <algorithm>
#include <cassert>

using namespace std;

class svlSortVectorDI {
public:
	vector<pair<double,int> > _myvec; // made public to make iterators easy to access manually
private:
	int _ix;
	bool _sorted, _ascending;
	vector<int> _orig;

public:
	svlSortVectorDI();
	svlSortVectorDI(bool ascending);
	svlSortVectorDI(int reserved_size);
	svlSortVectorDI(int reserved_size, bool ascending);
	virtual ~svlSortVectorDI();

	int size();
	void push_back(double val);
	void pop_back();
	void clear();
	void sort();
	void reorder();
	bool is_sorted();
	void reserve(int reserved_size);
	double get_value(int _ix);
	int get_ix(int _ix);
	int get_orig_ix(int _ix);
	double get_orig_val(int _ix);

	void setAscending(bool ascending);
	bool getAscending();
	void setDescending(bool descending);
	bool getDescending();

	// accessors
	double at(int _ix);
	double val_at(int _ix);
	int ix_at(int _ix);
	pair<double,int>& operator[](int _ix);
	pair<double,int>& back();

	void erase(int i);
};
