/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlTreeNodeT.h
** AUTHOR(S):   Stephen Gould <sgould@stanford.edu>
** DESCRIPTION:
**   Tree node struct used by kd-tree classes.
**
*****************************************************************************/

#pragma once

#include <vector>
#include "svlBoxT.h"

using namespace std;

template <class P>
struct svlTreeNodeT {
	vector<int> ix; // vector of indices referenced by node (size == 1 is node is branch, size > 1 if node is bucket)
	int dim; // the dim we split along
	double splitVal; // splitting 
	svlBoxT<P> box; // bounding box on 1) points in bucket if bucket node; 2) union of bounding boxes on child nodes if branch
	svlTreeNodeT *parent, *left, *right; // pointers to other nodes

	svlTreeNodeT()
		: dim(0), parent(NULL), left(NULL), right(NULL) {
		ix.reserve(1);
	}
	svlTreeNodeT(svlTreeNodeT *left, svlTreeNodeT *right)
		:  dim(0), parent(NULL), left(left), right(right) {
		ix.reserve(1);
	}
	svlTreeNodeT(int index)
		:  dim(0), parent(NULL), left(NULL), right(NULL) {
		ix.push_back(index); // push on one index
	}
	svlTreeNodeT(int index, svlTreeNodeT *left, svlTreeNodeT *right)
		:  dim(0), parent(NULL), left(left), right(right) {
		ix.push_back(index); // push on one index
	}
	svlTreeNodeT(int index, int dim, double splitVal)
		: dim(dim), splitVal(splitVal), parent(NULL), left(NULL), right(NULL) {
		ix.push_back(index); // push on one index
	}
	svlTreeNodeT(int index, int dim, double splitVal, svlTreeNodeT *left, svlTreeNodeT *right)
		: dim(dim), splitVal(splitVal), parent(NULL), left(left), right(right) {
		ix.push_back(index); // push on one index
	}
	~svlTreeNodeT() {
		ix.clear();
	}
};

