/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlBoostedClassifierSet.h
** AUTHOR(S):   Stephen Gould <sgould@stanford.edu>
** DESCRIPTION:
**  Implements a set (vector) of boosted classifiers as 1-vs-all binary
**  classifiers. Uses options:
**    boostingRounds : number of rounds of boosting
**    numSplits      : number of decision tree splits
**    trimRate       : boosting trim rate
**    pseudoCounts   : direchlet prior pseudo-counts
**
*****************************************************************************/

#pragma once

#include <vector>

#include "cv.h"
#include "cxcore.h"
#include "ml.h"

#include "svlBase.h"

using namespace std;

// svlBoostedClassifierSet class ---------------------------------------------

class svlBoostedClassifierSet : public svlOptions {
 protected:
    typedef struct _svlBoostedClassifierSetStats {
        int tp; int tn; int fp; int fn;
    } svlBoostedClassifierSetStats;

    vector<svlSmartPointer<CvBoost> > _models;

 public:
    svlBoostedClassifierSet(unsigned n = 0);
    svlBoostedClassifierSet(const svlBoostedClassifierSet& bcs);
    virtual ~svlBoostedClassifierSet();

    inline int size() const { return (int)_models.size(); }

    // i/o
    void initialize(unsigned n);
    bool save(const char *filestem, const char *ext = ".model", int modelIndx = -1);
    bool load(const char *filestem, const char *ext = ".model", int modelIndx = -1);

    // evaluation
    void evaluate(const vector<double>& x, vector<double>& y);
    void evaluate(const vector<vector<double> >& x,
	vector<vector<double> >& y);

    // train (returns average recall)
    double train(const vector<vector<double> >& x, const vector<int>& y,
        int modelIndx = -1);

    // validate (returns average recall)
    double validate(const vector<vector<double> >& x, const vector<int>& y,
        int modelIndx = -1);
   
 protected:
    svlBoostedClassifierSetStats scoreModel(int n,
        const vector<vector<double> >& x, const vector<int>& y);
};



