/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlFinalPatchDictionary.h
** AUTHOR(S):   Stephen Gould <sgould@stanford.edu>
**              Sid Batra <sidbatra@cs.stanford.edu>
**              Ian Goodfellow <ia3n@cs.stanford.edu>
**              Olga Russakovsky <olga@cs.stanford.edu>
**				Paul Baumstarck <pbaumstarck@stanford.edu>
** DESCRIPTION:
**   Stores a patch dictionary.
*****************************************************************************/


#pragma once

#include <cassert>
#include <iostream>
#include <vector>

#include "cv.h"
#include "cxcore.h"

#include "xmlParser/xmlParser.h"

#include "svlBase.h"
#include "svlVision.h"
//#include "svlDevel.h"

using namespace std;


// A smaller, struct version of svlIntensityPatchDefinition.
struct svlIntensityPatch {
	IplImage *patch;	// (Single-channel) patch template
	CvRect rect;		// Valid response region
	int channel;		// Channel information

	svlIntensityPatch() {}
	svlIntensityPatch(IplImage *patch, CvRect rect, int channel)
		: patch(patch), rect(rect), channel(channel) {}
	~svlIntensityPatch() {
		if ( patch ) cvReleaseImage(&patch);
	}
	void print() {
		for ( int i=0; i<patch->height; ++i ) {
			for ( int j=0; j<patch->width; ++j )
				printf(" %d", CV_IMAGE_ELEM(patch, unsigned char, i, j));
			cout << endl;
		}
	}
};

// A patch dictionary that just wraps its members without reference counting, virtual methods, or messy inheritance.
class svlFinalPatchDictionary {
public:
	CvSize _windowSize;						// Window size
    vector<svlIntensityPatch*> _intEntries;	// Intensity patches

public:
	svlFinalPatchDictionary(const char *file);
    svlFinalPatchDictionary(unsigned width = 0, unsigned height = 0);
    svlFinalPatchDictionary(const svlFinalPatchDictionary& d);
    virtual ~svlFinalPatchDictionary();

    void clear();
    bool read(const char *filename);
    bool read(XMLNode &root);
	inline int numFeatures() const {
		return int(_intEntries.size());
	}
	void print(); // Debug printing function.
    //pair<int, int> readClassifierSize(const char *filename);
};

